import re

# how to replace code and LaTeX blocks by HTML (<pre>) environment:
def HTML_code(filestr, format):
    c = re.compile(r'^!bc(.*?)\n', re.MULTILINE)
    filestr = c.sub(r'<!-- BEGIN VERBATIM BLOCK \g<1>-->\n<blockquote><pre>\n',
                    filestr)
    filestr = re.sub(r'!ec\n',
                     r'</pre></blockquote>\n<! -- END VERBATIM BLOCK -->\n',
                     filestr)
    c = re.compile(r'^!bt\n', re.MULTILINE)
    filestr = c.sub(r'<blockquote><pre>\n', filestr)
    filestr = re.sub(r'!et\n', r'</pre></blockquote>\n', filestr)
    return filestr

def define(FILENAME_EXTENSION,
           BLANKLINE,
           INLINE_TAGS_SUBST,
           CODE,
           LIST,
           ARGLIST,
           TABLE,
           FIGURE_EXT,
           INTRO,
           OUTRO):
    # all arguments are dicts and accept in-place modifications (extensions)
    
    FILENAME_EXTENSION['HTML'] = '.html'  # output file extension
    BLANKLINE['HTML'] = '<p>\n'           # blank input line => new paragraph

    INLINE_TAGS_SUBST['HTML'] = {         # from inline tags to HTML tags
        # keep math as is:
        'math': None,  # indicates no substitution
        'math2':         r'\g<begin>\g<puretext>\g<end>',
        'emphasize':     r'\g<begin><em>\g<subst></em>\g<end>',
        'bold':          r'\g<begin><b>\g<subst></b>\g<end>',
        'verbatim':      r'\g<begin><tt>\g<subst></tt>\g<end>',
        'label':         r'<a name="\g<subst>">',
        'reference':     r'<a href="#\g<subst>">\g<subst></a>',
        'linkURL':       r'\g<begin><a href="\g<url>">\g<link></a>\g<end>',
        'plainURL':      r'\g<begin><a href="\g<url>">\g<url></a>\g<end>',
        'section':       r'<h1>\g<subst></h1>',
        'subsection':    r'<h3>\g<subst></h3>',
        'subsubsection': r'<h5>\g<subst></h5>',
        'paragraph':     r'<b>\g<subst></b> ',
        'title':         r'<title>\g<subst></title>\n<center><h1>\g<subst></h1></center>',
        'date':          r'<center><h3>\g<subst></h3></center>',
        'author':        r'<center><h3>\g<subst></h3></center>',
        'figure':        r'<img src="\g<filename>" \g<options>> \g<caption>',
        'comment':       '<!-- %s -->',
        }

    CODE['HTML'] = HTML_code

    # how to typeset lists and their items in HTML:
    LIST['HTML'] = {
        'itemize':
        {'begin': '\n<ul>\n', 'item': '<li>', 'end': '</ul>\n\n'},

        'enumerate':
        {'begin': '\n<ol>\n', 'item': '<li>', 'end': '</ol>\n\n'},

        'description':
        {'begin': '\n<dl>\n', 'item': '<dt>%s<dd>', 'end': '</dl>\n\n'},

        'separator': '',  # no need for blank lines between items and before/after
        }

    # how to type set description lists for function arguments, return
    # values, and module/class variables:
    ARGLIST['HTML'] = {
        'parameter': '<b>argument</b>',
        'keyword': '<b>keyword argument</b>',
        'return': '<b>return value(s)</b>',
        'instance variable': '<b>instance variable</b>',
        'class variable': '<b>class variable</b>',
        'module variable': '<b>module variable</b>',
        }

    FIGURE_EXT['HTML'] = ('.png', '.gif', '.jpg', '.jpeg')

    # document start:
    INTRO['HTML'] = """
    <html>
    <body bgcolor="white">
    """
    # document ending:
    OUTRO['HTML'] = """
    </body>
    </html>
    """
