import os, commands, re

def LaTeX_code(filestr, format):
    c = re.compile(r'^!bc(.*?)\n', re.MULTILINE)
    filestr = c.sub(r'\\bccq\g<1>\n', filestr)
    filestr = re.sub(r'!ec\n', r'\\eccq\n', filestr)
    c = re.compile(r'^!bt\n', re.MULTILINE)
    filestr = c.sub('\n', filestr)
    filestr = re.sub(r'!et\n', '\n', filestr)
    return filestr

def figure_latex(m):
    filename = m.group('filename')
    basename  = os.path.basename(filename)
    stem, ext = os.path.splitext(basename)
    root, ext = os.path.splitext(filename)
    if not ext in ('.ps', '.eps'):
        # try to convert image file to PostScript, using
        # convert from ImageMagick:
        cmd = 'convert %s ps:%s.ps' % (filename, root)
        failure, output = commands.getstatusoutput(cmd)
        if failure:
            print '\n**** Warning: could not run', cmd
        filename = root + '.ps'
    # note that label{...} are substituted by \label{...} (inline
    # label tag) so we write just label and not \label below:
    result = r"""
\begin{figure}
  \centerline{\psfig{figure=%s,width=\linewidth}}
  \caption{label{fig:%s}
  %s
  }
\end{figure}
""" % (filename, stem, m.group('caption'))
    return result


def define(FILENAME_EXTENSION,
           BLANKLINE,
           INLINE_TAGS_SUBST,
           CODE,
           LIST,
           ARGLIST,
           TABLE,
           FIGURE_EXT,
           INTRO,
           OUTRO):
    # all arguments are dicts and accept in-place modifications (extensions)
    
    FILENAME_EXTENSION['LaTeX'] = '.ctex'
    BLANKLINE['LaTeX'] = '\n'

    INLINE_TAGS_SUBST['LaTeX'] = {
        # Note: re.sub "eats" backslashes: \t and \b will not survive to
        # latex if text goes through re.sub. Then we must write \\b and \\t here:

        # keep math as is:
        'math': None,  # indicates no substitution
        'math2':         r'\g<begin>$\g<latexmath>$\g<end>',        
        'emphasize':     r'\g<begin>\emph{\g<subst>}\g<end>',
        'bold':          r'\g<begin>\\textbf{\g<subst>}\g<end>',  # (re.sub swallows a \)
        'verbatim':      r'\g<begin>\code{\g<subst>}\g<end>',
        'label':         r'\label{\g<subst>}',
        'reference':     r'\\ref{\g<subst>}',
        'linkURL':       r'\g<begin>\href{\g<url>}{\g<link>}\g<end>',
        'plainURL':      r'\g<begin>\href{\g<url>}{\g<url>}\g<end>',
        'section':       '\n\n' + r'\section*{\g<subst>}' + '\n',
        'subsection':    '\n' + r'\subsection*{\g<subst>}' + '\n',
        'subsubsection': '\n' + r'\subsubsection*{\g<subst>}' + '\n',
        'paragraph':     r'\paragraph{\g<subst>}' + '\n',
        # recall that this is regex so LaTeX commands must be treated carefully:
        'title':         r'\\title{\g<subst>}' + '\n',
        'author':        r'\\author{\g<subst>}' + '\n',
        'date':          r'\\date{\g<subst>}' ' \n\\maketitle\n\n',
        'figure':        figure_latex,
        'comment':       '%% %s',
        }
    # should be configureable:
    # [tex]
    # verbatim = \code{, }
    # verbatim = \verb!, !

    LIST['LaTeX'] = {
        'itemize':
        {'begin': r'\begin{itemize}' + '\n',
         'item': r'\item', 'end': r'\end{itemize}' + '\n'},

        'enumerate':
        {'begin': r'\begin{enumerate}' + '\n', 'item': r'\item',
         'end': r'\end{enumerate}' + '\n'},

        'description':
        {'begin': r'\begin{description}' + '\n', 'item': r'\item[%s]',
         'end': r'\end{description}' + '\n'},

        'separator': '',
        } 

    CODE['LaTeX'] = LaTeX_code
    ARGLIST['LaTeX'] = {
    #    'parameter': r'\textbf{argument}',
    #    'keyword': r'\textbf{keyword argument}',
    #    'return': r'\textbf{return value(s)}',
    #    'instance variable': r'\textbf{instance variable}',
    #    'class variable': r'\textbf{class variable}',
    #    'module variable': r'\textbf{module variable}',
        'parameter': r'argument',
        'keyword': r'keyword argument',
        'return': r'return value(s)',
        'instance variable': r'instance variable',
        'class variable': r'class variable',
        'module variable': r'module variable',
        }

    FIGURE_EXT['LaTeX'] = ('.ps', '.eps')

    INTRO['LaTeX'] = r"""\documentclass{article}
\usepackage{hyperref,relsize,fancyvrb,psfig,a4}
\begin{document}
"""
    OUTRO['LaTeX'] = r"""\end{document}
"""

